#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const yargs_1 = __importDefault(require("yargs"));
const inquirer_1 = __importDefault(require("inquirer"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const questions_1 = __importDefault(require("./questions"));
const utils_1 = require("./utils");
const yargsOpts = Object.fromEntries(questions_1.default.map((_a) => {
    var { name, type, choices } = _a, rest = __rest(_a, ["name", "type", "choices"]);
    const value = Object.assign(Object.assign({ type: type === 'confirm' ? 'boolean' : type === 'checkbox' ? 'array' : type === 'number' ? 'number' : 'string' }, choices ? { choices: choices.map((c) => c.value) } : {}), rest);
    return [name, value];
}));
(() => __awaiter(void 0, void 0, void 0, function* () {
    const _a = yield yargs_1.default.options(yargsOpts).argv, { _: [name], $0 } = _a, initialValues = __rest(_a, ["_", "$0"]);
    const _b = yield inquirer_1.default.prompt(questions_1.default, Object.assign(Object.assign({ name }, initialValues), { ci: optionsProvided(Object.assign(Object.assign({}, initialValues), { name })) })), { options: o } = _b, answers = __rest(_b, ["options"]);
    const options = Object.assign(Object.assign({}, answers), Object.fromEntries(o.map((v) => [v, true])));
    // create folder for project
    const projectFolder = path_1.default.resolve(process.cwd(), `./${options.name}`);
    if (fs_1.default.existsSync(projectFolder)) {
        console.error(`The directory ${options.name} already exists`);
        process.exit(1);
    }
    fs_1.default.mkdirSync(projectFolder, { recursive: true });
    // generate package.json and write it to the new folder
    const json = yield (0, utils_1.generatePackageJson)(options);
    fs_1.default.writeFileSync(path_1.default.resolve(process.cwd(), `./${options.name}/package.json`), JSON.stringify(json, null, 2));
    // run npm install in the project folder
    /* TODO: temporarily turned off for testing
    child_process.execSync(`npm --prefix ./${options.name} install ./${options.name}`, { stdio: [0, 1, 2] })
    */
    // construct files
    const filesDir = path_1.default.resolve(__dirname, './files');
    const snippetsDir = path_1.default.resolve(__dirname, './snippets');
    const files = fs_1.default.readdirSync(filesDir);
    files.forEach(fileName => {
        var _a;
        const fileContents = fs_1.default.readFileSync(`${filesDir}/${fileName}`, 'utf8');
        const file = JSON.parse(fileContents);
        if (!(0, utils_1.filterRules)(file.rules, options)) {
            return;
        }
        let imports = file.imports || [];
        const filePath = path_1.default.resolve(process.cwd(), `./${options.name}/${file.path}`);
        const fileDirname = path_1.default.dirname(filePath);
        const content = file.content && !file.snippets ? JSON.stringify(file.content, null, 2) : (_a = file.snippets) === null || _a === void 0 ? void 0 : _a.filter(snippet => (0, utils_1.filterRules)(snippet.rules, options)).map(snippet => {
            const snippetContent = snippet.file ? fs_1.default.readFileSync(`${snippetsDir}/${snippet.file}`, 'utf8') : '';
            imports = imports.concat(snippetContent.match(/^import .*$/gm) || [], snippet.imports || []);
            return snippetContent
                .replace(new RegExp(snippet.replace ? Object.keys(snippet.replace).map(key => key).join('|') : '$^', 'gi'), (match) => match ? options[snippet.replace[match]] : '')
                .replace(/^(declare|import) .*$/gm, '');
        }).join('\r\n').replace(/[\r\n]\s{2,}[\r\n]/g, '\r\n\r\n');
        if (content || imports.length > 0) {
            fs_1.default.mkdirSync(fileDirname, { recursive: true });
            fs_1.default.writeFileSync(filePath, `${imports.join('\r\n')}${content}`);
        }
    });
}))();
function optionsProvided(options) {
    return Boolean(options.auth) && Boolean(options.name);
}
//# sourceMappingURL=index.js.map